import { render, unmountComponentAtNode } from "react-dom";
import SensorData from "../routes/SensorData";
import { screen } from "@testing-library/react";

let conteneur = null;
beforeEach(() => {
  // met en place un élément DOM comme cible de rendu
  conteneur = document.createElement("div");
  document.body.appendChild(conteneur);
});

afterEach(() => {
  // nettoie en sortie de test
  try {
    unmountComponentAtNode(conteneur);
  } catch (ignored) {}
  conteneur.remove();
  conteneur = null;
});

const interrupteur_off = JSON.parse(`{
  "id": 45678,
  "name": "Interrupteur cuisine",
  "type": "SWITCH",
  "data": {
      "value": 0
  }
}`);

it("Generation interrupteur off", () => {
  render(<SensorData sensor={interrupteur_off} />, conteneur);
  expect(screen.getByText("L'interrupteur est off")).toBeInTheDocument();
  expect(screen.getByAltText("Tourner off interrupteur")).toBeInTheDocument();
});

const interrupteur_on = JSON.parse(`{
    "id": 45678,
    "name": "Interrupteur cuisine",
    "type": "SWITCH",
    "data": {
        "value": 1
    }
}`);

it("Generation interrupteur on", () => {
  render(<SensorData sensor={interrupteur_on} />, conteneur);
  expect(screen.getByText("L'interrupteur est on")).toBeInTheDocument();
  expect(screen.getByAltText("Tourner on interrupteur")).toBeInTheDocument();
});

const lumiere_eteinte = JSON.parse(`{
  "id": 34561,
  "name": "Lumière salon",
  "type": "LIGHT",
  "data": {
      "value": 0
  }
}`);

it("Generation lumiere eteinte", () => {
  render(<SensorData sensor={lumiere_eteinte} />, conteneur);
  expect(screen.getByText("La lumière est off")).toBeInTheDocument();
  expect(screen.getByAltText("Tourner off ampoule")).toBeInTheDocument();
});

const lumiere_allumer = JSON.parse(`{
  "id": 34561,
  "name": "Lumière salon",
  "type": "LIGHT",
  "data": {
      "value": 50
  }
}`);

it("Generation light on", () => {
  render(<SensorData sensor={lumiere_allumer} />, conteneur);
  expect(
    screen.getByText("La lumière est avec une intensité de 50%")
  ).toBeInTheDocument();
  expect(screen.getByAltText("Tourner on ampoule")).toBeInTheDocument();
});

const Porte_Fermer = JSON.parse(`{
  "id": 10245,
  "name": "Porte du Garage",
  "type": "DOOR",
  "data": {
      "value": 0
  }
}`);

it("Generation closed door", () => {
  render(<SensorData sensor={Porte_Fermer} />, conteneur);
  expect(screen.getByText("La porte est fermer")).toBeInTheDocument();
  expect(screen.getByAltText("Porte fermer")).toBeInTheDocument();
});

const Porte_ouverte = JSON.parse(`{
  "id": 10245,
  "name": "Porte du Garage",
  "type": "DOOR",
  "data": {
      "value": 1
  }
}`);

it("Generation opened", () => {
  render(<SensorData sensor={Porte_ouverte} />, conteneur);
  expect(screen.getByText("La porte est ouverte")).toBeInTheDocument();
  expect(screen.getByAltText("Porte ouverte")).toBeInTheDocument();
});

const Sensor_Humidité = JSON.parse(`{
  "id": 48569,
  "name": "Humidité SDB",
  "type": "HUMIDITY",
  "data": {
      "labels": [
          "2022-03-26T05:00:57.341Z",
          "2022-03-26T05:10:57.341Z",
          "2022-03-26T05:20:57.341Z",
          "2022-03-26T05:30:57.341Z",
          "2022-03-26T05:40:57.341Z",
          "2022-03-26T05:50:57.341Z",
          "2022-03-26T06:00:57.341Z",
          "2022-03-26T06:10:57.341Z",
          "2022-03-26T06:20:57.341Z",
          "2022-03-26T06:30:57.341Z",
          "2022-03-26T06:40:57.341Z",
          "2022-03-26T06:50:57.341Z",
          "2022-03-26T07:00:57.341Z",
          "2022-03-26T07:10:57.341Z",
          "2022-03-26T07:20:57.341Z",
          "2022-03-26T07:30:57.341Z",
          "2022-03-26T07:40:57.341Z",
          "2022-03-26T07:50:57.341Z",
          "2022-03-26T08:00:57.341Z"
      ],
      "values": [
          38,
          38,
          41,
          40,
          40,
          41,
          40,
          42,
          50,
          65,
          60,
          53,
          45,
          40,
          39,
          38,
          39,
          38,
          38
      ]
  }
}`);

it("Generation humidity", () => {
  render(<SensorData sensor={Sensor_Humidité} />, conteneur);
  expect(screen.getByTestId("chart")).toBeInTheDocument();
});

const Sensor_temperature = JSON.parse(`{
  "id": 1234,
  "name": "Température Bureau",
  "type": "TEMPERATURE",
  "data": {
      "labels": [
          "2021-01-19T08:00:00.000Z",
          "2021-01-19T09:00:00.000Z",
          "2021-01-19T10:00:00.000Z",
          "2021-01-19T11:00:00.000Z",
          "2021-01-19T12:00:00.000Z",
          "2021-01-19T13:00:00.000Z",
          "2021-01-19T14:00:00.000Z",
          "2021-01-19T15:00:00.000Z",
          "2021-01-19T16:00:00.000Z"
      ],
      "values": [
          23,
          23,
          22,
          21,
          23,
          23,
          23,
          25,
          25
      ]
  }
}`);

it("Generation temperature", () => {
  render(<SensorData sensor={Sensor_temperature} />, conteneur);
  expect(screen.getByTestId("chart")).toBeInTheDocument();
});

const ventilateur = JSON.parse(`{
  "id": 2222,
  "name": "Ventilateur Ordinateur Bureau",
  "type": "FAN_SPEED",
  "data": {
      "labels": [
          "2021-01-19T10:00:00.000Z",
          "2021-01-19T10:05:00.000Z",
          "2021-01-19T10:10:00.000Z",
          "2021-01-19T10:15:00.000Z",
          "2021-01-19T10:20:00.000Z",
          "2021-01-19T10:25:00.000Z"
      ],
      "values": [
          1073,
          1800,
          2299,
          2176,
          1899,
          1400
      ]
  }
}`);

it("Generation ventilateur", () => {
  render(<SensorData sensor={ventilateur} />, conteneur);
  expect(screen.getByTestId("chart")).toBeInTheDocument();
});

const invalide = JSON.parse(`{
  "id": "0000",
  "name": "Invalid sensor",
  "type": "Unknown",
  "data": ["Invalid"]
}`);

it("invalid sensor", () => {
  render(<SensorData sensor={invalide} />, conteneur);
  expect(screen.getByTestId("invalidSensor")).toBeInTheDocument();
});

const Sensor_Sans_Donnée = JSON.parse(`{
  "id": "0000",
  "name": "Invalid sensor",
  "type": "Unknown"
}`);

it("sensor without data", () => {
  render(<SensorData sensor={Sensor_Sans_Donnée} />, conteneur);
  expect(screen.getByTestId("noDataSensor")).toBeInTheDocument();
});

